#include <libnova.h>
#include <curses.h>
#include <time.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <sys/types.h>
#include <math.h>

void usage(char * name)
{    
 
    printf(" Usage: %s [-g longitude] [-l latitude] [-u delay]\n",name);
    printf(" -g: longitude (degrees 0.0-90.0 Negative East of Greenwinch, positive West of Greenwinch)\n");
    printf(" -l: latitude  (degrees 0.0-180.0)\n");
    printf(" -v: Print version information and exit\n");
    printf(" -u: delay (0 - 10, default 4): Screen update delay\n\n");
}



double moonphase() {
double phase,age,julian_day;
const char * fmt="%Y";
const char * fmt2="%d";
const char * fmt3="%m";
struct tm *gm;
time_t t;
long year, month, day, k1, k2, k3;

time(&t);
gm = localtime(&t);
year = gm->tm_year + 1900;
month = gm->tm_mon + 1;
day = gm->tm_mday;
year = year - (int)(( 12 - month ) / 10);
month = month + 9;
if (month >= 12) {
month = month - 12;
                 }
k1 = (int)(365.25 * ( year + 4712 ));
k2 = (int)(30.6 * month + 0.5 );
k3 = (int)((int)((year / 100) + 49) * 0.75) - 38;
julian_day = k1 + k2 + day + 59;
if (julian_day > 2299160) {
julian_day = julian_day - k3;
			  }
phase = (julian_day - 2451550.1) / 29.53058868;
phase = phase - (int)(phase);
if (phase < 0) {
phase = phase + 1;
}

age = phase * 29.53;
phase = phase * 2.0;
if (phase > 1.99) {
phase = 0.0;
}
if (phase > 1.00) {
phase = 1.00 - (phase - 1.00);
}
phase = (int)(phase * 100.0);
return age;

} /* end moonphase */




void version(char * name)
{   
    
printf("%s version 1.0 by mulo (mulo@makemebad.net) www: http://makemebad.net/geek/codes/\n",name);
			 
}

void finish(int sigage) 
{
    curs_set(1);
    clear();
    refresh();
    resetty();
    endwin();
    exit(0);
}


int calculate (int update,double latitude,double longitude) {
double julian,light,dist,phase,age;
struct ln_lnlat_posn observer;
struct ln_rst_time rst;
struct ln_date date1,date2,date3;
time_t tempo;
struct tm * tempostruct;
const char * fmt="%Y";
const char * fmt2="%d";
size_t maxsize=100;
char giorno[maxsize];
char anno[maxsize];
int row,cols;

memset(&rst, 0, sizeof(rst));
memset(&date1, 0, sizeof(date1));
memset(&date2, 0, sizeof(date2));
memset(&date3, 0, sizeof(date3));

while (1) {
julian=get_julian_from_sys();
age=moonphase();


if ( (latitude != 0) || (longitude != 0 ) ) {
observer.lng=latitude;
observer.lat=longitude;
} else {observer.lng=45.12;observer.lat=8.42;}
light=100*get_lunar_disk(julian);
dist=get_lunar_earth_dist(julian);
get_lunar_rst(julian,&observer,&rst);
get_date(rst.rise,&date1);
get_date(rst.set,&date2);
get_date(julian,&date3);
getmaxyx(stdscr,row,cols);

mvprintw(row/4,0,
"
        Moon's situation at: %d/%d/%d %d:%d:%d
        Age: +- %d days
        Rise: %d/%d/%d %d:%d:%d
        Set: %d/%d/%d %d:%d:%d
        Illuminated: %.4f%%
        Moon->Earth distance: %.3f Km\n\n
",
date3.days,date3.months,date3.years,date3.hours,date3.minutes,(int)date3.seconds,(int)age,
date1.days,date1.months,date1.years,date1.hours,date1.minutes,(int)date1.seconds,
date2.days,date2.months,date2.years,date2.hours,date2.minutes,(int)date2.seconds,light,dist);
napms(update * 1000);
refresh();
}




}


int main (int argc, char ** argv) {
int row,cols,optchr,update=1;
double latitude=0,longitude=0;

 signal(SIGINT, finish);
 while ((optchr = getopt(argc, argv, "vhu:l:g:")) != EOF) 
{

	switch (optchr) {
	case 'h':
    	usage(argv[0]);
    	exit(0);
	case 'v':
    	version(argv[0]);    
    	exit(0);
	case 'u':
        update = atoi(optarg);
        break;
        case 'l':
        latitude = atoi(optarg);
        break; 
        case 'g':
        longitude = atoi(optarg);    
        break;
        	         	}
}

  initscr();
  savetty();
  nonl();
  cbreak();
  noecho(); 
  getmaxyx(stdscr,row,cols);
  leaveok(stdscr, TRUE);
  curs_set(0);
  calculate(update,latitude,longitude);



					}

